@@ -31,7 +31,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: accessctl.c,v 1.14 2005/04/29 23:09:02 hb9xar Exp $";
+static char const ident[]="$Id: accessctl.c,v 1.15 2006/05/20 11:48:53 hb9xar Exp $";
 
 /* configuration storage */
 struct siproxd_config configuration;
@@ -167,9 +167,9 @@ int process_aclist (char *aclist, struct sockaddr_in from) {
       mask_int=atoi(mask);
       bitmask= (mask_int)? (0xffffffff<<(32-mask_int)) : 0;
 
-      DEBUGC(DBCLASS_ACCESS,"[%i] (%p) <-> (%p)", i,
-                            ntohl(inaddr.s_addr) & bitmask,
-			    ntohl(from.sin_addr.s_addr) & bitmask);
+      DEBUGC(DBCLASS_ACCESS,"check match: entry=%i, filter=%lx, from=%lx", i,
+                            (long)ntohl(inaddr.s_addr) & bitmask,
+			    (long)ntohl(from.sin_addr.s_addr) & bitmask);
 
       if ( (ntohl(inaddr.s_addr) & bitmask) == 
            (ntohl(from.sin_addr.s_addr) & bitmask) ) return STS_SUCCESS;
