@@ -43,7 +43,7 @@
 #include "rewrite_rules.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c,v 1.38 2005/05/05 08:49:59 hb9xar Exp $";
+static char const ident[]="$Id: sip_utils.c,v 1.45 2006/05/20 11:50:51 hb9xar Exp $";
 
 
 /* configuration storage */
@@ -84,6 +84,14 @@ osip_message_t *msg_make_template_reply (sip_ticket_t *ticket, int code) {
    osip_to_clone (request->to, &response->to);
    osip_from_clone (request->from, &response->from);
 
+   /* if 3xx, also include 1st contact header */
+   if ((code==200) || ((code>=300) && (code<400))) {
+      osip_contact_t *req_contact = NULL;
+      osip_contact_t *res_contact = NULL;
+      osip_message_get_contact(request, 0, &req_contact);
+      if (req_contact) osip_contact_clone (req_contact, &res_contact);
+      if (res_contact) osip_list_add(response->contacts,res_contact,0);
+   }
 
    /* via headers */
    pos = 0;
@@ -517,7 +525,7 @@ int sip_gen_response(sip_ticket_t *ticket, int code) {
    osip_via_t *via;
    int port;
    char *buffer;
-   int buflen;
+   size_t buflen;
    struct in_addr addr;
 
    /* create the response template */
@@ -655,51 +663,74 @@ int sip_del_myvia (sip_ticket_t *ticket) {
 int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
    osip_message_t *sip_msg=ticket->sipmsg;
    osip_contact_t *contact;
-   int i;
+   int i, j;
+   int replaced=0;
 
    if (sip_msg == NULL) return STS_FAILURE;
 
+   /* at least one contact header present? */
    osip_message_get_contact(sip_msg, 0, &contact);
    if (contact == NULL) return STS_FAILURE;
 
-   for (i=0;i<URLMAP_SIZE;i++){
-      if (urlmap[i].active == 0) continue;
-      if ((direction == DIR_OUTGOING) &&
-          (compare_url(contact->url, urlmap[i].true_url)==STS_SUCCESS)) break;
-      if ((direction == DIR_INCOMING) &&
-          (compare_url(contact->url, urlmap[i].masq_url)==STS_SUCCESS)) break;
-   }
+   /* loop for all existing contact headers in message */
+   for (j=0; contact != NULL; j++) {
+      osip_message_get_contact(sip_msg, j, &contact);
+      if (contact == NULL) break;
+      if (contact->url == NULL) continue;
 
-   /* found a mapping entry */
-   if (i<URLMAP_SIZE) {
-      char *tmp;
-      DEBUGC(DBCLASS_PROXY, "rewrote Contact header %s@%s -> %s@%s",
-             (contact->url->username)? contact->url->username : "*NULL*",
-             (contact->url->host)? contact->url->host : "*NULL*",
-             urlmap[i].masq_url->username, urlmap[i].masq_url->host);
-
-      /* remove old entry */
-      osip_list_remove(sip_msg->contacts,0);
-      osip_contact_to_str(contact, &tmp);
-      osip_contact_free(contact);
-
-      /* clone the url from urlmap*/
-      osip_contact_init(&contact);
-      osip_contact_parse(contact,tmp);
-      osip_free(tmp);
-      osip_uri_free(contact->url);
-      if (direction == DIR_OUTGOING) {
-         /* outgoing, use masqueraded url */
-         osip_uri_clone(urlmap[i].masq_url, &contact->url);
-      } else {
-         /* incoming, use true url */
-         osip_uri_clone(urlmap[i].true_url, &contact->url);
+      /* search for an entry */
+      for (i=0;i<URLMAP_SIZE;i++){
+         if (urlmap[i].active == 0) continue;
+         if ((direction == DIR_OUTGOING) &&
+             (compare_url(contact->url, urlmap[i].true_url)==STS_SUCCESS)) break;
+         if ((direction == DIR_INCOMING) &&
+             (compare_url(contact->url, urlmap[i].masq_url)==STS_SUCCESS)) break;
       }
 
-      osip_list_add(sip_msg->contacts,contact,-1);
-   } else {
+      /* found a mapping entry */
+      if (i<URLMAP_SIZE) {
+         char *tmp;
+
+         if (direction == DIR_OUTGOING) {
+            DEBUGC(DBCLASS_PROXY, "rewriting Contact header %s@%s -> %s@%s",
+                   (contact->url->username)? contact->url->username : "*NULL*",
+                   (contact->url->host)? contact->url->host : "*NULL*",
+                   urlmap[i].masq_url->username, urlmap[i].masq_url->host);
+         } else {
+            DEBUGC(DBCLASS_PROXY, "rewriting Contact header %s@%s -> %s@%s",
+                   (contact->url->username)? contact->url->username : "*NULL*",
+                   (contact->url->host)? contact->url->host : "*NULL*",
+                   urlmap[i].true_url->username, urlmap[i].true_url->host);
+         }
+
+         /* remove old entry */
+         osip_list_remove(sip_msg->contacts,j);
+         osip_contact_to_str(contact, &tmp);
+         osip_contact_free(contact);
+
+         /* clone the url from urlmap*/
+         osip_contact_init(&contact);
+         osip_contact_parse(contact,tmp);
+         osip_free(tmp);
+         osip_uri_free(contact->url);
+         if (direction == DIR_OUTGOING) {
+            /* outgoing, use masqueraded url */
+            osip_uri_clone(urlmap[i].masq_url, &contact->url);
+         } else {
+            /* incoming, use true url */
+            osip_uri_clone(urlmap[i].true_url, &contact->url);
+         }
+
+         osip_list_add(sip_msg->contacts,contact,j);
+         replaced=1;
+      }
+
+   }
+
+   if (replaced == 0) {
+      DEBUGC(DBCLASS_PROXY, "no Contact header rewritten");
       return STS_FAILURE;
-   } 
+   }
 
    return STS_SUCCESS;
 }
@@ -945,6 +976,173 @@ int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
 }
 
 
+/*
+ * SIP_IS_OUTGOING
+ *
+ * Figures out if this is an outgoing or incoming request/response.
+ * The direction is stored in the ticket->direction property.
+ *
+ * RETURNS
+ *	STS_SUCCESS on success
+ *	STS_FAILURE if unable to determine
+ */
+int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
+   int type;
+   int i, sts;
+   struct sockaddr_in *from;
+   osip_message_t *request;
+   osip_message_t *response;
+
+   from=&ticket->from;
+   request=ticket->sipmsg;
+   response=ticket->sipmsg;
+   type = 0;
+
+   ticket->direction = 0;
+
+   /*
+    * did I receive the telegram from a REGISTERED host?
+    * -> it must be an OUTGOING request
+    */
+   for (i=0; i<URLMAP_SIZE; i++) {
+      struct in_addr tmp_addr;
+
+      if (urlmap[i].active == 0) continue;
+      if (get_ip_by_host(urlmap[i].true_url->host, &tmp_addr) == STS_FAILURE) {
+         DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve host [%s]",
+             urlmap[i].true_url->host);
+      } else {
+         DEBUGC(DBCLASS_SIP, "sip_find_direction: reghost:%s ip:%s",
+                urlmap[i].true_url->host, utils_inet_ntoa(from->sin_addr));
+         if (memcmp(&tmp_addr, &from->sin_addr, sizeof(tmp_addr)) == 0) {
+            if (MSG_IS_REQUEST(ticket->sipmsg)) {
+               type=REQTYP_OUTGOING;
+            } else {
+               type=RESTYP_OUTGOING;
+            }
+            break;
+         }
+      }
+   }
+
+   /*
+    * is the telegram directed to an internally registered host?
+    * -> it must be an INCOMING request
+    */
+   if (type == 0) {
+      for (i=0; i<URLMAP_SIZE; i++) {
+         if (urlmap[i].active == 0) continue;
+         /* RFC3261:
+          * 'To' contains a display name (Bob) and a SIP or SIPS URI
+          * (sip:bob@biloxi.com) towards which the request was originally
+          * directed.  Display names are described in RFC 2822 [3].
+          */
+
+         /* So this means, that we must check the SIP URI supplied with the
+          * INVITE method, as this points to the real wanted target.
+          * Q: does there exist a situation where the SIP URI itself does
+          *    point to "somewhere" but the To: points to the correct UA?
+          * So for now, we just look at both of them (SIP URI and To: header)
+          */
+
+         if (MSG_IS_REQUEST(ticket->sipmsg)) {
+            /* REQUEST */
+            /* incoming request (SIP URI == 'masq') || ((SIP URI == 'reg') && !REGISTER)*/
+            if ((compare_url(request->req_uri, urlmap[i].masq_url)==STS_SUCCESS) ||
+                (!MSG_IS_REGISTER(request) &&
+                 (compare_url(request->req_uri, urlmap[i].reg_url)==STS_SUCCESS))) {
+               type=REQTYP_INCOMING;
+               break;
+            }
+            /* incoming request ('to' == 'masq') || (('to' == 'reg') && !REGISTER)*/
+            if ((compare_url(request->to->url, urlmap[i].masq_url)==STS_SUCCESS) ||
+                (!MSG_IS_REGISTER(request) &&
+                 (compare_url(request->to->url, urlmap[i].reg_url)==STS_SUCCESS))) {
+               type=REQTYP_INCOMING;
+               break;
+            }
+         } else { 
+            /* RESPONSE */
+            /* incoming response ('from' == 'masq') || ('from' == 'reg') */
+            if ((compare_url(response->from->url, urlmap[i].reg_url)==STS_SUCCESS) ||
+                (compare_url(response->from->url, urlmap[i].masq_url)==STS_SUCCESS)) {
+               type=RESTYP_INCOMING;
+               break;
+            }
+         } /* is request */
+      } /* for i */
+   } /* if type == 0 */
+
+
+   if (MSG_IS_RESPONSE(ticket->sipmsg)) {
+      /* &&&& Open Issue &&&&
+         it has been seen with cross-provider calls that the FROM may be 'garbled'
+         (e.g 1393xxx@proxy01.sipphone.com for calls made sipphone -> FWD)
+         How can we deal with this? Should I take into consideration the 'Via'
+         headers? This is the only clue I have, pointing to the *real* UA.
+         Maybe I should put in a 'siproxd' ftag value to recognize it as a header
+         inserted by myself
+      */
+      if ((type == 0) && (!osip_list_eol(response->vias, 0))) {
+         osip_via_t *via;
+         struct in_addr addr_via, addr_myself;
+         int port_via, port_ua;
+
+         /* get the via address */
+         via = (osip_via_t *) osip_list_get (response->vias, 0);
+         DEBUGC(DBCLASS_SIP, "sip_find_direction: check via [%s] for "
+                "registered UA",via->host);
+         sts=get_ip_by_host(via->host, &addr_via);
+         if (sts == STS_FAILURE) {
+            DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve VIA [%s]",
+                   via->host);
+         } else {
+
+            for (i=0; i<URLMAP_SIZE; i++) {
+               if (urlmap[i].active == 0) continue;
+               /* incoming response (1st via in list points to a registered UA) */
+               sts=get_ip_by_host(urlmap[i].true_url->host, &addr_myself);
+               if (sts == STS_FAILURE) {
+                  DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve "
+                         "true_url [%s]", via->host);
+                  continue;
+               }
+
+               port_via=0;
+               if (via->port) port_via=atoi(via->port);
+               if (port_via <= 0) port_via=SIP_PORT;
+
+               port_ua=0;
+               if (urlmap[i].true_url->port)
+                  port_ua=atoi(urlmap[i].true_url->port);
+               if (port_ua <= 0) port_ua=SIP_PORT;
+
+               DEBUGC(DBCLASS_SIP, "sip_find_direction: checking for registered "
+                      "host [%s:%i] <-> [%s:%i]",
+                      urlmap[i].true_url->host, port_ua,
+                      via->host, port_via);
+
+               if ((memcmp(&addr_myself, &addr_via, sizeof(addr_myself))==0) &&
+                   (port_via == port_ua)) {
+                  type=RESTYP_INCOMING;
+                  break;
+               }
+            } /* for i */
+         }
+      } /* if type == 0 */
+   } /* is response */
+
+   if (type == 0) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: unable to determine "
+                          "direction of SIP packet");
+      return STS_FAILURE;
+   }
+
+   ticket->direction=type;
+   if (urlidx) *urlidx=i;
+   return STS_SUCCESS;
+}
+
 
 
 
